@@ -3,11 +3,9 @@
 from functools import wraps
 
 from trytond.model import ModelView, Workflow, fields
-from trytond.transaction import Transaction
-from trytond.pool import PoolMeta, Pool
+from trytond.pool import Pool, PoolMeta
 from trytond.pyson import Eval
-
-from trytond.modules.product import round_price
+from trytond.transaction import Transaction
 
 
 class Location(metaclass=PoolMeta):
@@ -18,7 +16,6 @@ class Location(metaclass=PoolMeta):
         states={
             'invisible': ~Eval('type').in_(['supplier', 'storage']),
             },
-        depends=['type'],
         help="The party invoiced when consignment stock is used.")
 
     @classmethod
@@ -67,7 +64,7 @@ def set_origin_consignment(func):
         to_save = []
         move2line = {}
         for move in moves:
-            if all(l.origin != move for l in move.invoice_lines):
+            if not move.consignment_invoice_lines:
                 lines = move.get_invoice_lines_consignment()
                 if lines:
                     to_save.extend(lines)
@@ -92,12 +89,11 @@ def unset_origin_consignment(func):
         InvoiceLine = pool.get('account.invoice.line')
         lines, to_save = [], []
         for move in moves:
-            for invoice_line in move.invoice_lines:
-                if invoice_line.origin == move:
-                    lines.append(invoice_line)
-                    if move.origin == move:
-                        move.origin = None
-                    to_save.append(move)
+            for invoice_line in move.consignment_invoice_lines:
+                lines.append(invoice_line)
+                if move.origin == move:
+                    move.origin = None
+                to_save.append(move)
         if lines:
             InvoiceLine.delete(lines)
             cls.save(to_save)
@@ -108,6 +104,13 @@ def unset_origin_consignment(func):
 class Move(metaclass=PoolMeta):
     __name__ = 'stock.move'
 
+    consignment_invoice_lines = fields.One2Many(
+        'account.invoice.line', 'origin', "Consignment Invoice Lines",
+        readonly=True,
+        states={
+            'invisible': ~Eval('consignment_invoice_lines'),
+            })
+
     @fields.depends('state', 'from_location', 'to_location')
     def on_change_with_unit_price_required(self, name=None):
         required = super().on_change_with_unit_price_required(name)
@@ -140,19 +143,24 @@ class Move(metaclass=PoolMeta):
                 required = True
         return required
 
-    def _get_tax_rule_pattern(self):
-        return {}
+    @property
+    def is_supplier_consignment(self):
+        return (self.from_location.type == 'supplier'
+            and self.to_location.type in {'storage', 'production', 'customer'}
+            and self.from_location.consignment_party)
+
+    @property
+    def is_customer_consignment(self):
+        return (
+            self.from_location.type in {'storage', 'production', 'supplier'}
+            and self.to_location.type == 'customer'
+            and self.from_location.consignment_party)
 
     def get_invoice_lines_consignment(self):
         lines = []
-        if (self.from_location.type == 'supplier'
-                and self.to_location.type in {
-                    'storage', 'production', 'customer'}
-                and self.from_location.consignment_party):
+        if self.is_supplier_consignment:
             lines.append(self._get_supplier_invoice_line_consignment())
-        if (self.from_location.type in {'storage', 'production', 'supplier'}
-                and self.to_location.type == 'customer'
-                and self.from_location.consignment_party):
+        if self.is_customer_consignment:
             lines.append(self._get_customer_invoice_line_consignment())
         return lines
 
@@ -178,27 +186,11 @@ class Move(metaclass=PoolMeta):
         line.party = self.from_location.consignment_party
         line.currency = currency
         line.product = self.product
-        line.description = self.product.name
         line.quantity = self.quantity
-        line.unit = self.uom
-        line.account = self.product.account_expense_used
+        line.unit = self.unit
         line.stock_moves = [self]
         line.origin = self
-
-        taxes = set()
-        pattern = self._get_tax_rule_pattern()
-        for tax in line.product.supplier_taxes_used:
-            if line.party.supplier_tax_rule:
-                tax_ids = line.party.supplier_tax_rule.apply(tax, pattern)
-                if tax_ids:
-                    taxes.update(tax_ids)
-                continue
-            taxes.add(tax.id)
-        if line.party.supplier_tax_rule:
-            tax_ids = line.party.supplier_tax_rule.apply(None, pattern)
-            if tax_ids:
-                taxes.update(tax_ids)
-        line.taxes = taxes
+        line.on_change_product()
 
         with Transaction().set_context(
                 currency=line.currency.id,
@@ -207,7 +199,6 @@ class Move(metaclass=PoolMeta):
                 taxes=[t.id for t in line.taxes]):
             line.unit_price = Product.get_purchase_price(
                 [line.product], line.quantity)[line.product.id]
-            line.unit_price = round_price(line.unit_price)
         return line
 
     def _get_customer_invoice_line_consignment(self):
@@ -222,27 +213,11 @@ class Move(metaclass=PoolMeta):
         line.party = self.from_location.consignment_party
         line.currency = self.company.currency
         line.product = self.product
-        line.description = self.product.name
         line.quantity = self.quantity
-        line.unit = self.uom
-        line.account = self.product.account_revenue_used
+        line.unit = self.unit
         line.stock_moves = [self]
         line.origin = self
-
-        taxes = set()
-        pattern = self._get_tax_rule_pattern()
-        for tax in line.product.customer_taxes_used:
-            if line.party.customer_tax_rule:
-                tax_ids = line.party.customer_tax_rule.apply(tax, pattern)
-                if tax_ids:
-                    taxes.update(tax_ids)
-                continue
-            taxes.add(tax.id)
-        if line.party.customer_tax_rule:
-            tax_ids = line.party.customer_tax_rule.apply(None, pattern)
-            if tax_ids:
-                taxes.update(tax_ids)
-        line.taxes = taxes
+        line.on_change_product()
 
         with Transaction().set_context(
                 currency=line.currency.id,
@@ -251,7 +226,6 @@ class Move(metaclass=PoolMeta):
                 taxes=[t.id for t in line.taxes]):
             line.unit_price = Product.get_sale_price(
                 [line.product], line.quantity)[line.product.id]
-            line.unit_price = round_price(line.unit_price)
         return line
 
     @classmethod
